/*
 * SerializableShipDesc.cpp
 *
 * Defines a ship selected with FleetPick.
 *
 * (C) 2001 MagnumMan-iCoP (James E. King, III)
 */

#ifndef _ssd
#define _ssd

#define SSD_REVISION	2		// Used to make sure struct changes are noticed

class SerializableShipDesc
{
public:

	char	shipCode[16];
	int		bpv_base;
	int		bpv_curr;			// Current adjusted BPV

	int		marines;			// number of marines the ship should have at startup
	int		mines;				// number of mines the ship should have at startup
	int		repairs;			// number of repairs the ship should have at startup

	int		shuttles;			// number of shuttles the ship should have at startup

	int		drone_size;			// 1 = Type I (small); 4 = Type IV (heavy)
	int		drone_speed;		// 1 = slow; 2 = medium; 3 = large
	int		drone_reloads;		// 1 = 1 reload, ..., 4 = 4 reloads
	int		drone_count;		// more useful for the API

	int		fighter_bays;
	int		fighter_count;		// per bay
	char	fighter_type[40];
	
	SerializableShipDesc()
	{
		Reset();
	}

	~SerializableShipDesc()
	{
	}

	inline void Reset(void)
	{
		memset(this, 0, sizeof(SerializableShipDesc));
	}
	
	inline bool Write(FILE *fp)
	{
		return fwrite(this, sizeof(SerializableShipDesc), 1, fp) == 1;
	}

	inline bool Read(FILE *fp)
	{
		return fread(this,  sizeof(SerializableShipDesc), 1, fp) == 1;
	}

	inline void	Dump(FILE *fp)
	{
		fprintf(fp, "SSD Dump:\n");
		fprintf(fp, "Ship: %s BPV %d Adjusted %d\n", shipCode, bpv_base, bpv_curr);

		fprintf(fp, "Shuttles: %d\n Marines: %d\n   Mines: %d\n Repairs: %d\n",
			shuttles, marines, mines, repairs);

		fprintf(fp, "Drone size: %d\n      speed: %d\n     reloads: %d\n", drone_size, drone_speed, drone_reloads);

		fprintf(fp, "Fighter Bays: %d\n", fighter_bays);
		fprintf(fp, "    Per  Bay: %dx%s\n", fighter_count, fighter_type);
	}		
};

#endif
