#include "stdafx.h"

#include "CoopData.h"
#include <Windows.h>

CoopData gCoopData;

CoopData::CoopData()
{
	memset(this, 0, sizeof(CoopData));
}

CoopData::~CoopData()
{
}

iTruth CoopData::FindSFCOPDirectory(std::string &location)
{
	HKEY	hKey;
	LONG	lval;
	DWORD	type;
	char	path[256];
	DWORD	siz = 256;

	lval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
						"Software\\Taldren\\Starfleet Command Orion Pirates",
						0,
						KEY_QUERY_VALUE,
						&hKey);

	if (lval != ERROR_SUCCESS)
		return kFalse;

	lval = RegQueryValueEx(	hKey,
							"Directory",
							0,
							&type,
							(unsigned char *)path,
							&siz );

	RegCloseKey(hKey);

	if (lval != ERROR_SUCCESS)
		return kFalse;

	path[siz] = 0;
	if (path[siz - 1] != '\\')
		strcat(path, "\\");

	location = path;
	return kTrue;
}

iTruth CoopData::FindScriptsDirectory(std::string &location)
{
	iTruth t = FindSFCOPDirectory(location);
	if (t == kTrue)
		location.append("Assets\\Scripts\\");

	return t;
}

iTruth	CoopData::Load(void)
{
	char buf[64];
	std::string	scriptsINI;
	if (!FindScriptsDirectory(scriptsINI))
		return kFalse;
	scriptsINI += "scripts.ini";

	DWORD	rv = GetPrivateProfileString(
					"Mul_CoopAce", "multiplier",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		fEnemyBPVRatio = atof(buf);
	else
		fEnemyBPVRatio = 1.1f;			// DEFAULT: 1.10

	rv = GetPrivateProfileString(
					"Mul_CoopAce", "race",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		fRaces = atoi(buf);
	else
		fRaces = 0;						// DEFAULT: Random

	rv = GetPrivateProfileString(
					"Mul_CoopAce", "hull",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		fShipClasses = atoi(buf);
	else
		fShipClasses = 0xFFFFFFFF;		// DEFAULT: Enable All Hulls

	std::string	sfcINI;
	if (!FindSFCOPDirectory(sfcINI))
		return kFalse;
	sfcINI += "sfc.ini";

	rv = GetPrivateProfileString(
					"UI", "UIType",
					"", buf, 64,
					sfcINI.c_str());

	if (rv)
		fAIIntel = atof(buf);
	else
		fAIIntel = 0;					// NO DEFAULT...? Probably CAPTAIN = 0

	return kTrue;
}

iTruth CoopData::Save(void)
{
	char buf[64];

	std::string	scriptsINI;
	if (!FindScriptsDirectory(scriptsINI))
		return kFalse;
	scriptsINI += "scripts.ini";

	std::string	sfcINI;
	if (!FindSFCOPDirectory(sfcINI))
		return kFalse;
	sfcINI += "sfc.ini";

	sprintf(buf, "%1.2f", fEnemyBPVRatio);
	if (!WritePrivateProfileString("Mul_CoopAce", "multiplier",
								   buf, scriptsINI.c_str()))
		return kFalse;
	
	sprintf(buf, "%d", fRaces);
	if (!WritePrivateProfileString("Mul_CoopAce", "race",
								   buf, scriptsINI.c_str()))
		return kFalse;
	
	sprintf(buf, "%d", fShipClasses);
	if (!WritePrivateProfileString("Mul_CoopAce", "hull",
								   buf, scriptsINI.c_str()))
		return kFalse;

	sprintf(buf, "%d", fAIIntel);
	if (!WritePrivateProfileString("UI", "UIType",
								   buf, sfcINI.c_str()))
		return kFalse;

	return kTrue;
}

float	CoopData::mGetEnemyBPVRatio(void) { return fEnemyBPVRatio; }
void	CoopData::mSetEnemyBPVRatio(float f) { fEnemyBPVRatio = f; }

void	CoopData::mResetRaceSelections(void) { fRaces = 0; }
iTruth	CoopData::mIsRaceRandom(void) { return fRaces == 0; }
iTruth	CoopData::mIsRaceSelected(eRace r) { return (fRaces & (1 << r)) != 0; }
void	CoopData::mSetRaceSelected(eRace r) { fRaces |= (1 << r); }

void	CoopData::mResetShipClassSelections(void) { fShipClasses = 0; }
iTruth	CoopData::mIsShipClassSelected(eClassTypes c) { return (fShipClasses & (1 << c)) != 0; }
void	CoopData::mSetShipClassSelected(eClassTypes c) { fShipClasses |= (1 << c); }

int32u	CoopData::mGetRawRaceSelected(void) { return fRaces; }
int32u	CoopData::mGetRawShipClassSelected(void) { return fShipClasses; }

int32u	CoopData::mGetAIIntel(void) { return fAIIntel; }
void    CoopData::mSetAIIntel(int32u i) { fAIIntel = i; }
