/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonDefs.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "scriptinterface.h"
#include "CoopCustomizer/CoopData.h"
#include "ShipList.h"
#include "Mul_CoopAce.h"

tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

static bool	FindScriptsDirectory(std::string &location)
{
	HKEY	hKey;
	LONG	lval;
	DWORD	type;
	char	path[256];
	DWORD	siz = 256;

	lval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
						"Software\\Taldren\\Starfleet Command Orion Pirates",
						0,
						KEY_QUERY_VALUE,
						&hKey);

	if (lval != ERROR_SUCCESS)
		return false;

	lval = RegQueryValueEx(	hKey,
							"Directory",
							0,
							&type,
							(unsigned char *)path,
							&siz );

	RegCloseKey(hKey);

	if (lval != ERROR_SUCCESS)
		return false;

	path[siz] = 0;
	if (path[siz - 1] != '\\')
		strcat(path, "\\");

	strcat(path, "Assets\\Scripts\\");

	location = path;
	return true;
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	if (!gScriptInterface.mIsHost())
		return;							// not host, doesn't matter
	
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPC1 ),
					nPCBPV,
					kClassFrigate,
					kClassFrigate,
					0,
					kStartPosition_Z,
					kPlayerNPC1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPC2 ), "PL3" , kStartPosition_J, kNoMetaShipID, kPlayerNPC2 );

*/

void tEnemyTeam::mCreateShipsForTeam()
{
	gCoopData.Load();
	mCreateCoopEnemy();					// was called from a timer but I didn't like the one-time lag
}

static int	bpv_to_assign = 0;
ShipList	carry_over_sl;
int32	earliestShip = 999;
int32	latestShip = -999;

#define		MAX_ASSIGN	8

void tEnemyTeam::mCreateCoopEnemy()		// called from timer
{
	if (!gScriptInterface.mIsHost())
		return;							// not host, doesn't matter
	
	FILE *		fp;
	ShipList	sl;

	if (mGetTeam() == kTeam7)
		fp = fopen("C:\\Coop.txt", "w");
	else	
		fp = fopen("C:\\Coop.txt", "a");

	if (mGetTeam() == kTeam7)
	{
		sl.CreateMaster(ShipList::kOrionPirates, false);

		fprintf(fp, "First Enemy Team (kTeam7): initializing Co-op System...\n");
		fflush(fp);

		std::string	scriptsINI;

		FindScriptsDirectory(scriptsINI);
		scriptsINI.append("scripts.ini");

		fprintf(fp, "host set bpv multiplier = %1.2f\n", gCoopData.mGetEnemyBPVRatio());
		fflush(fp);
		fprintf(fp, "host set races = %d\n", gCoopData.mGetRawRaceSelected());
		fflush(fp);
		fprintf(fp, "host set hulls = %d\n", gCoopData.mGetRawShipClassSelected());
		fflush(fp);

		int32	totalPlayerBPV = 0;
		int32	totalShipCount = 0;
		int32	thisTeamShipCount = 0;

		for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
		{
			thisTeamShipCount = 0;

			tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)team);
			if (ti)
			{
				tShipIterator tsi = ti->mGetFirstShip();
				for (;;)
				{
					tShipInfo *si = *tsi;
					if (!si)
						break;

				// mGetClassName returns invalid pointer here
				//	fprintf(fp, "Found a ship belonging to team %d: '%s'\n" /* bpv %d.\n" */, ((int)team) + 1, si->mGetClassName()); //, si->mGetBPV());
				//	fflush(fp);

					thisTeamShipCount++;
				//	totalPlayerBPV += si->mGetBPV(); doesn't work, always returns 0
				//	totalPlayerBPV += si->mGetTotalBPV(); doesn't work, always returns 0
					tsi++;
				}

				totalShipCount += thisTeamShipCount;

				if (thisTeamShipCount)
				{
					fprintf(fp, "Team %d has %d ships totalling %d combat BPV (not including loadouts).\n", ((int)team) + 1, thisTeamShipCount,
						ti->mGetCombatBPV());
				//	totalPlayerBPV += ti->mGetTotalCombatBPV(); doesn't work, always returns 0
					totalPlayerBPV += ti->mGetCombatBPV(); // doesn't include loadouts :<
				}
			}
		}

		fprintf(fp, "\nLoadout costs cannot be determined by scripts.  Bugs... :<\n\n");

		fprintf(fp, "Total Player BPV is %d.\n\n", totalPlayerBPV);
		fflush(fp);

		totalPlayerBPV *= gCoopData.mGetEnemyBPVRatio();
		bpv_to_assign = totalPlayerBPV;

		fprintf(fp, "Total Enemy BPV is %d.\n\n", totalPlayerBPV);
		fflush(fp);

		/*
		 * Calculate era and limit ships to that era and earlier
		 */

		Ship s;
		fprintf(fp, "Calculating game era...\n");
		for (PlayerShipClassNameList::iterator pi = pscnl.begin(); pi != pscnl.end(); pi++)
		{
			if (sl.Find(*pi, s))
			{
				fprintf(fp, "PlayerShipClassName %s (yfa=%d yla=%d) ", pi->c_str(), s.yearFirstAvail, s.yearLastAvail);
				fflush(fp);

				if (s.yearFirstAvail > 36) fprintf(fp, "ADVANCED\n");
				else if (s.yearFirstAvail > 7) fprintf(fp, "LATE\n");
				else if (s.yearFirstAvail >= 0) fprintf(fp, "MID\n");
				else fprintf(fp, "EARLY\n");
				fflush(fp);
				
				if (s.yearFirstAvail < earliestShip)
					earliestShip = s.yearFirstAvail;
				if (s.yearFirstAvail > latestShip)
					latestShip = s.yearFirstAvail;
			}
		}
		
		if (latestShip > 36)
		{
			fprintf(fp, "advanced era detected - all ships available to picker\n");
			fflush(fp);
		}
		else if (latestShip > 7)
		{			// late era - do nothing
			fprintf(fp, "late era detected - stripping off advanced ships\n");
			fflush(fp);
			sl.Subset(ShipList::kSubsetYearAvail, 37, 99999, false);
		}
		else if (latestShip >= 0)
		{			// mid era - strip off late
			fprintf(fp, "mid era detected - stripping off late and advanced ships\n");
			fflush(fp);
			sl.Subset(ShipList::kSubsetYearAvail, 8, 99999, false);
		}
		else
		{			// early era - strip off mid, late
			fprintf(fp, "early era detected - stripping off mid, late, and advanced ships\n");
			fflush(fp);
			sl.Subset(ShipList::kSubsetYearAvail, 0, 99999, false);
		}
		fprintf(fp, "\n");

		std::list< eRace >	races;

		if (gCoopData.mIsRaceRandom())
		{
			races.push_back((eRace)fMissionInfo->mRandomInt32(kFederation, kOrion));
				// was through kMonster, if kMonster then monster ship class required
				// decided to remove kMonster possibility from "single race".
			fprintf(fp, "Race chosen: %d\n", (int)(*(races.begin())));
			fflush(fp);
		}
		else
		{
			if (gCoopData.mIsRaceSelected(kFederation))	{ races.push_back(kFederation); fprintf(fp, "Race Federation Available.\n"); }
			if (gCoopData.mIsRaceSelected(kKlingon))	{ races.push_back(kKlingon);	fprintf(fp, "Race Klingon Available.\n"); }
			if (gCoopData.mIsRaceSelected(kRomulan))	{ races.push_back(kRomulan);	fprintf(fp, "Race Romulan Available.\n"); }
			if (gCoopData.mIsRaceSelected(kGorn))		{ races.push_back(kGorn);		fprintf(fp, "Race Gorn Available.\n"); }
			if (gCoopData.mIsRaceSelected(kLyran))		{ races.push_back(kLyran);		fprintf(fp, "Race Lyran Available.\n"); }
			if (gCoopData.mIsRaceSelected(kHydran))		{ races.push_back(kHydran);		fprintf(fp, "Race Hydran Available.\n"); }
			if (gCoopData.mIsRaceSelected(kISC))		{ races.push_back(kISC);		fprintf(fp, "Race ISC Available.\n"); }
			if (gCoopData.mIsRaceSelected(kMirak))		{ races.push_back(kMirak);		fprintf(fp, "Race Mirak Available.\n"); }
			if (gCoopData.mIsRaceSelected(kMonster))	{ races.push_back(kMonster);	fprintf(fp, "Race Monster Available.\n"); }
			if (gCoopData.mIsRaceSelected(kOrion))		{ races.push_back(kOrionOrion);
														  races.push_back(kOrionKorgath);
														  races.push_back(kOrionPrime);
														  races.push_back(kOrionTigerHeart);
														  races.push_back(kOrionBeastRaiders);
														  races.push_back(kOrionSyndicate);
														  races.push_back(kOrionWyldeFire);
														  races.push_back(kOrionCamboro);
																						fprintf(fp, "Race Orion Pirates Available.\n"); }
		}

		sl.Subset(races);

		std::list< eClassTypes >	classes;

		if (gCoopData.mIsShipClassSelected(kClassFrigate))				{ classes.push_back(kClassFrigate);				fprintf(fp, "Hull Frigate Available.\n"); }
		if (gCoopData.mIsShipClassSelected(kClassDestroyer))			{ classes.push_back(kClassDestroyer);			fprintf(fp, "Hull Destroyer Available.\n"); }
		if (gCoopData.mIsShipClassSelected(kClassWarDestroyer))			{ classes.push_back(kClassWarDestroyer);		fprintf(fp, "Hull War Destroyer Available.\n"); }
		if (gCoopData.mIsShipClassSelected(kClassLightCruiser))			{ classes.push_back(kClassLightCruiser);		fprintf(fp, "Hull Light Cruiser Available.\n"); }
		if (gCoopData.mIsShipClassSelected(kClassHeavyCruiser))			{ classes.push_back(kClassHeavyCruiser);		fprintf(fp, "Hull Heavy Cruiser Available.\n"); }
		if (gCoopData.mIsShipClassSelected(kClassNewHeavyCruiser))		{ classes.push_back(kClassNewHeavyCruiser);		fprintf(fp, "Hull New Heavy Cruiser Available.\n"); }
		if (gCoopData.mIsShipClassSelected(kClassHeavyBattlecruiser))	{ classes.push_back(kClassHeavyBattlecruiser);	fprintf(fp, "Hull Heavy Battlecruiser Available.\n"); }
		if (gCoopData.mIsShipClassSelected(kClassCarrier))				{ classes.push_back(kClassCarrier);				fprintf(fp, "Hull Carrier Available.\n"); }
		if (gCoopData.mIsShipClassSelected(kClassDreadnought))			{ classes.push_back(kClassDreadnought);			fprintf(fp, "Hull Dreadnought Available.\n"); }
		if (gCoopData.mIsShipClassSelected(kClassBattleship))			{ classes.push_back(kClassBattleship);			fprintf(fp, "Hull Battleship Available.\n"); }
		if (gCoopData.mIsRaceSelected(kMonster))						{ classes.push_back(kClassMonster);				fprintf(fp, "Hull Monster Available.\n"); }
		
		sl.Subset(classes);

		fprintf(fp, "\nShips Available:\n\n");
		fflush(stdout);

		fprintf(fp, "Code      BPV  YFA YLA\n");
		SHIP_LIST_ITER(sl)
			fprintf(fp, "%-8s %4d %4d %3d\n",
				iter->shipCode.c_str(), iter->bpv, iter->yearFirstAvail, iter->yearLastAvail);
		SHIP_LIST_ITER_END
		fprintf(fp, "\n");
		fflush(fp);
		carry_over_sl = sl;
	}
	else if (mGetTeam() == kTeam8)
	{
		fprintf(fp, "Second Enemy Team (kTeam8): continuing where kTeam7 left off...\n");
		fflush(fp);
	}
	else if (mGetTeam() == kTeam9)
	{
		fprintf(fp, "Third Enemy Team (kTeam9): continuing where kTeam8 left off...\n");
		fflush(fp);
	}
	else if (mGetTeam() == kTeam10)
	{
		fprintf(fp, "Fourth Enemy Team (kTeam10): continuing where kTeam9 left off...\n");
		fflush(fp);
	}

	//
	// This code is executed for all of the enemy teams.
	// When the first team has MAX_ASSIGN ships, it spills over into
	// the next team.  When it runs out of ships selection is over.
	// We do this because if you put more than 8 ships on an enemy team
	// the engine seems to die.
	//

	ShipList	mylist = carry_over_sl;
	int			ships_can_assign = MAX_ASSIGN;

	while (bpv_to_assign > 0 && mylist.size() && ships_can_assign > 0)
	{
		ShipList	worklist = mylist;
		Ship		s;
		
		fprintf(fp, "BPV left to assign: %d\n", bpv_to_assign);
		fflush(fp);

		// preference to pick inside era
		if (fMissionInfo->mRandomInt32(0, 100) < 80)
		{
			fprintf(fp, "in-era preferred choice selector activated\n");

			if (latestShip > 36)
			{
				worklist.Subset(ShipList::kSubsetYearAvail, 37, 99999, true);
				if (!worklist.size())
				{
					printf("in-era selector detected no ships in era; using all\n");
					worklist = mylist;
				}
			}
			else if (latestShip > 7)
			{
				worklist.Subset(ShipList::kSubsetYearAvail, 8, 36, true);
				if (!worklist.size())
				{
					printf("in-era selector detected no ships in era; using all\n");
					worklist = mylist;
				}
			}
			else if (latestShip >= 0)
			{
				worklist.Subset(ShipList::kSubsetYearAvail, 0, 7, true);
				if (!worklist.size())
				{
					printf("in-era selector detected no ships in era; using all\n");
					worklist = mylist;
				}
			}
			else
				;	// early era list will only have early era.
		}
		
		// preference towards larger ships at higher bpvs
ta:		if (bpv_to_assign > 250 && fMissionInfo->mRandomInt32(0, 100) < 65)
		{
			fprintf(fp, "Activated big ship choice code.\n");

			ShipList bigsl = worklist;
			std::list< eClassTypes >	classes;
			if (gCoopData.mIsShipClassSelected(kClassHeavyBattlecruiser))	classes.push_back(kClassHeavyBattlecruiser);
			if (gCoopData.mIsShipClassSelected(kClassCarrier))				classes.push_back(kClassCarrier);
			if (gCoopData.mIsShipClassSelected(kClassDreadnought))			classes.push_back(kClassDreadnought);
			if (gCoopData.mIsShipClassSelected(kClassBattleship))			classes.push_back(kClassBattleship);
			if (gCoopData.mIsRaceSelected(kMonster))						classes.push_back(kClassMonster);
			bigsl.Subset(classes);
			if (!bigsl.SelectRandomShip(s))
				continue;				// There's no big ship left; continue on
		}
		else
		{
			if (!worklist.SelectRandomShip(s))
				break;
		} 

		int32 tmpval = bpv_to_assign - s.bpv;

		if (tmpval < 0)
		{
			mylist.Remove(s.shipCode);		// Not enough BPV left for this ship
		//	continue;						// so remove from list and pick again
		// optimization...
			worklist.Remove(s.shipCode);
			goto ta;
		}

		if (tmpval > 14 && tmpval < 50 && mylist.size() > 1)
		{
			mylist.Remove(s.shipCode);		// This ship choice would leave too little
		//	continue;						// bpv for another selection and there is a
		// optimization...
			worklist.Remove(s.shipCode);
			goto ta;
		}									// "possibility" for a better pick
		
		bpv_to_assign -= s.bpv;

		fprintf(fp, "Creating enemy ship (Team %d/Ship %d) '%s' value %d\n", mGetTeam() + 1, MAX_ASSIGN - ships_can_assign, s.shipCode.c_str(), s.bpv);
		fflush(fp);
		
		mCreateShip( typeid( tShipInfo ),
					 s.shipCode.c_str(),
					 kStartPosition_M,
					 kNoMetaShipID,
					 -1, 0, 0, -1, -1, NULL, kDefaultShipOptions);
	}

	carry_over_sl = mylist;		// carry over the pared down list for expedience
	fprintf(fp, "Enemy setup complete.\n\n");
	fclose(fp);
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

