/********************************************************************************************************\

	File Name:	HumanTeamBaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "HumanTeamBaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "worldconditions.h"

//
//	Serialization events
//
tHumanTeamBaseVictoryState::tHumanTeamBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tHumanTeamBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tHumanTeamBaseVictoryState::mBeginState()
{}
void tHumanTeamBaseVictoryState::mEndState()
{}

eVictoryLevels tHumanTeamBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
//	fMissionInfo->mDebugMessage("tHumanTeamBaseVictoryState::mCalcVictoryStatus");

	if (gWC.mGetDidWin())
	{
//		fMissionInfo->mDebugMessage("kAstoundingVictory");
		return kAstoundingVictory;
	}
	else if (gWC.mGetGameEnded())
	{
//		fMissionInfo->mDebugMessage("kLeftEarly_msg");
		return kUnknownVictoryLevel;
	}
	else
	{
//		fMissionInfo->mDebugMessage("kDevastatingDefeat");
		return kDevastatingDefeat;
	}
}

void tHumanTeamBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

//	fMissionInfo->mDebugMessage("tHumanTeamBaseVictoryState::mOnDeath");

	if ( !fChildActor->mGetTeam()->mShipsCurrentlyControlled() )
	{
//		fMissionInfo->mDebugMessage("no ships in control, team destroyed");
		gScriptInterface.mSetGameStatus( fChildActor->mGetTeam()->mGetTeam(), kPlayerDestroyed );
	}

	mSetTimedInterval(kTimerSecondsSet, 1, 1.0f);
}

void tHumanTeamBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

//	fMissionInfo->mDebugMessage("tHumanTeamBaseVictoryState::mOnCapture");

	if ( !fChildActor->mGetTeam()->mShipsCurrentlyControlled() )
	{
		fMissionInfo->mDebugMessage("no ships in control, team captured");
		gScriptInterface.mSetGameStatus( fChildActor->mGetTeam()->mGetTeam(), kPlayerCaptured );
	}

	mSetTimedInterval(kTimerSecondsSet, 1, 1.0f);
}

void tHumanTeamBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

//	fMissionInfo->mDebugMessage("tHumanTeamBaseVictoryState::mOnDisengaged");

	if ( fChildActor->mGetTeam()->mShipsCurrentlyControlled() )
	{
//		fMissionInfo->mDebugMessage("no ships in control, team disengaged");
		gScriptInterface.mSetGameStatus( fChildActor->mGetTeam()->mGetTeam(), kPlayerDisengaged );
	}

	mSetTimedInterval(kTimerSecondsSet, 1, 1.0f);
}

void tHumanTeamBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tHumanTeamBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tHumanTeamBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tHumanTeamBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tHumanTeamBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tHumanTeamBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tHumanTeamBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tHumanTeamBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tHumanTeamBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tHumanTeamBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tHumanTeamBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tHumanTeamBaseVictoryState::mOnFlashingButtonPressed()
{}
void tHumanTeamBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tHumanTeamBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tHumanTeamBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tHumanTeamBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tHumanTeamBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tHumanTeamBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tHumanTeamBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//

/*

	HELPER NOTE:	When this event is called, you need to pretend that the team associated with the
					playerNumber has been defeated.  So call what you need to, so that you may determine
					if the game has ended.  For instance, a world condition?

*/
void tHumanTeamBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{
//	fMissionInfo->mDebugMessage("tHumanTeamBaseVictoryState::mOnPlayerLeaving");
	gWC.mCheckForGameEnd();
}
void tHumanTeamBaseVictoryState::mOnHostChanged()
{
//	fMissionInfo->mDebugMessage("tHumanTeamBaseVictoryState::mOnHostChanged");
	gWC.mCheckForGameEnd();
}
void tHumanTeamBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	gWC.mCheckForGameEnd();
}
void tHumanTeamBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tHumanTeamBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


