/********************************************************************************************************\

	Mission:	Mul_CoopAce
	Script:		"Mul_CoopAce"
	Descript:	<DESCRIPTION>

	Created by:		<NAME>			mm/dd/yy
	Last Updated:	<INITIALS>		mm/dd/yy
					<LOG>


\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "HumanTeam.h"
#include "EnemyTeam.h"
#include "Mul_CoopAce.h"
#include "VictoryCondition.h"

DEFINE_SCRIPT_CLASS( tMul_CoopAce );

PlayerShipClassNameList	pscnl;

tMul_CoopAce::tMul_CoopAce()
{
	srand(GetTickCount());
}

/*

	HELPER NOTE:	Below set up the mission specifications for this script.  Each type of mission can require
					various fields to be set.  These types are as follows:

					kCampaignMission
						- for general Metaverse missions
					kUniqueCampaignMission
						- for "elite" Metaverse missions
						- cannot have kPrimaryOpponentTeam or kPlayableTeam
					kTutorialMission
						- the fWait field determines where this script appears in the list
					kMultiplayerMission
						- cannot have kPrimaryTeam or kPrimaryOppenentTeam

*/
void tMul_CoopAce::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fHelpTags = Messages[kHelpTags_msg];
	Specs->fMissionType = kMultiplayerMission;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecAllShips;
	Specs->fMenuStates = kEnableSupplyDock | kDisableAI | kDisableTagB | kDisableTagC | kDisableTagD | kDisableTagE | kDisableTagF;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k1Band | k5Client );

	/*

		HELPER NOTE:	In this case, there are 6 teams setup.  Feel free to change the number of teams
						by deleting them starting with team6 and working your way down to team3, inclusive.
						Also, if the team is to be controlled by the AI and only the AI, then pass the
						constant 'kNPCTeam' instead of 'kPlayableTeam' into the team specification.  Also,
						not noted here, the last parameter of mCreateTeamSpec() can be a team tag such as
						"kTagA" to "kTagF".  This is the default tag for a kNPCTeam that was not chosen
						through the multiplayer UI screen.

	*/
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1",
												kTeam1,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tHumanTeam ),
												kTagA);

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2",
												kTeam2,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tHumanTeam ),
												kTagA);

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3",
												kTeam3,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tHumanTeam ),
												kTagA);

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4",
												kTeam4,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tHumanTeam ),
												kTagA);

	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5",
												kTeam5,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tHumanTeam ),
												kTagA);

	const tTeamSpec& team6 = mCreateTeamSpec(	"Team 6",
												kTeam6,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tHumanTeam ),
												kTagA);

	const tTeamSpec& team7 = mCreateTeamSpec(	"Enemy Block 1",		// Ships 1-10
												kTeam7,
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tEnemyTeam ),
												kTagB);

	const tTeamSpec& team8 = mCreateTeamSpec(	"Enemy Block 2",		// Ships 11-20
												kTeam8,
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tEnemyTeam ),
												kTagB);

	const tTeamSpec& team9 = mCreateTeamSpec(	"Enemy Block 3",		// Ships 21-30
												kTeam9,
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tEnemyTeam ),
												kTagB);

	const tTeamSpec& team10 = mCreateTeamSpec(	"Enemy Block 4",		// Ships 31-40
												kTeam10,
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tEnemyTeam ),
												kTagB);

	/*

		HELPER NOTE:	Please note that although you setup the teams, you need to create ships and their
						respective files.  Otherwise, an assert will occur.  It is up to you to decide
						whether the ship files should be shared by multiple physical teams or just a
						1-to-1 relationship between the team and the ship files.

	*/
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( team7 );
	Specs->mSetupTeam( team8 );
	Specs->mSetupTeam( team9 );
	Specs->mSetupTeam( team10 );
}

/*

	HELPER NOTE:	Return the map index you want to use for only kUniqueCampaignMission.

*/
int32 tMul_CoopAce::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when all ships have been created for the script.

*/
void tMul_CoopAce::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	kCampaignMission may require an overall briefing in the future, at which point, you would
					want to return the briefing index.

*/
int32 tMul_CoopAce::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tMul_CoopAce::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you would normally set up the teams' relationships and goals.  However,
					since this is a multiplayer mission you will set up the relations between team tags
					rather than the teams themselves.

*/
void tMul_CoopAce::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 2;
	int32 tagList[kRelationsCount] =
	{
		kTagA,
		kTagB
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 A     B
		/*A   */ "AI", "WD",
		/*B   */ "WD", "AI",
	};

	fMissionInfo->mSetTagRelations( tagList, ( std::string* )relationships, kRelationsCount );

	/*

		HELPER NOTE:	You may wish to check if the game has ended here in the case, for instance,
						that all physical teams share the same team tag letter and therefore, have
						no common enemies.

	*/
}

/*

	HELPER NOTE:	Since you really do not know which teams are going to be associated with which team
					tags until the game starts, you must create the teams here.  This function may look
					like the following:

						void tMul_CoopAce::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
						{
							switch ( teamScriptDescription.fTeamTag )
							{
							case kTagA:
							case kTagB:
							case kTagC:
							case kTagD:
							case kTagE:
							case kTagF:
								{
									mMission()->mSetupTeam(	teamScriptDescription.fTeam,
															teamScriptDescription.fTeamTag,
															teamScriptDescription.fRace,
															teamScriptDescription.fHumanControlled,
															typeid( tCommonTeam ) );
								}
								break;
							}
						}

					If you do not care about player-chosen tags, then you can just keep this function the
					way it is; otherwise, overload this implementation as explained above.

*/
void tMul_CoopAce::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
{
	tScript::mNewTeam( teamScriptDescription );
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.
					Your custom values may look something like the following:

*/
void tMul_CoopAce::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
/*	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['J'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['K'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['L'] = kObjectPlayerShip; */

	Map->fTacticalMapIcon['M'] = kObjectEnemyShip;
}
