/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "missioninfo.h"

/*

	HELPER NOTE:	This global reference to the world condition is here twofold.  One, it can be easily
					accessed from the all victory states; two, it maintains the consistancy relatively
					easily throughout the script.

*/
class tWorldConditions gWC;

/*

	HELPER NOTE:	Remember to instantiate your world conditions here.

*/
tWorldConditions::tWorldConditions()
{}


/*

	HELPER NOTE:	Acts as a sort of victory condition for all of the teams.  This is where you can
					analyze the game state according to the world conditions you have kept.  It is recommended
					that you call ::mSetGameStatus() here, when a victory decision has been made for all teams.

*/
void tWorldConditions::mCheckForGameEnd( void )
{
	int32	playerShipCount = 0;
	int32	enemyShipCount = 0;
	int32	team;

	if (!gScriptInterface.mIsHost())
		return;							// not host, doesn't matter
	
	//
	// Check to see if there are any enemy ships left.
	// If none left, player wins.
	//

	for (team = (int)kTeam1; team <= (int)kTeam10; team++)
	{
		tTeamInfo *ti = gScriptInterface.mMission()->mGetTeamHandle((eTeamID)team);
		if (ti)
		{
			int32 i = ti->mGetShipsInPlayCount();		// can be -1... odd!


			//
			// If ship captured, must check current team
			//

			for (int k = 0; k < i; k++)
			{
				tShipInfo *	tsi = (tShipInfo *)ti->mGetThisShip(k+1);
				int32 shipTeam = tsi->mGetTeam();
				if (shipTeam >= (int)kTeam1 && shipTeam <= (int)kTeam6)
					playerShipCount++;
				else
					enemyShipCount++;
			}
		}
	}

	if (enemyShipCount <= 0)
	{
		mSetGameEnded(kTrue);
		mSetDidWin(kTrue);

		// All enemy ships are gone

		for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
			gScriptInterface.mSetGameStatus((eTeamID) team, kPlayerWins);
	}

	if (playerShipCount <= 0)
	{
		mSetGameEnded(kTrue);
		mSetDidWin(kFalse);

		// All player ships are gone

		for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
			gScriptInterface.mSetGameStatus((eTeamID) team, kPlayerLoses);
	}
}
