/********************************************************************************************************\

	File Name:	MissionMaps.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionMaps.h"

/*
	G-L are fixed starting positions for the 6 teams
*/
int32 TotalMaps = 12;
const char* MapTypes[] = {	"Open Space - L",
							"Open Space - M",
							"Open Space - S",
							"Asteroid Field - L",
							"Asteroid Field - M",
							"Asteroid Field - S",
							"Asteroid Belt - L",
							"Asteroid Belt - M",
							"Asteroid Belt - S",
							"Nebula - L",
							"Nebula - M",
							"Nebula - S" };

/*
	Legend:
			Starting Positions			Ending Positions			Planets			
			Start1 --> G				End1 --> g					Earth --> 1
			Start2 --> H				End2 --> h					Ringed Earth --> 2
			Start3 --> I				End3 --> i					Mars --> 3
			Start4 --> J				End4 --> j					Jupiter --> 4
			Start5 --> K				End5 --> k					Organia --> 5
			Start6 --> L				End6 --> l					Saturn --> 6
			Start7 --> M				End7 --> m					Night City --> 7
			Start8 --> N				End8 --> n					Fire --> 8
			Start9 --> O				End9 --> o					Ice --> 9
			Start10 --> P				End10 --> p					Gas --> 0
			Start11 --> Q				End11 --> q					
			Start12 --> R				End12 --> r					
			Start13 --> S				End13 --> s					
			Start14 --> T				End14 --> t					
			Start15 --> U				End15 --> u					
			Start16 --> V				End16 --> v					
			Start17 --> W				End17 --> w					
			Start18 --> X				End18 --> x					
			Start19 --> Y				End19 --> y					
			Start20 --> Z				End20 --> z					

			Celestial Bodies			Asteroids					Black Holes
			Moon --> @					SmallAsteroid --> [			SmallBlackHole --> :
			Sun1 --> (					LargeAsteroid --> ]			LargeBlackHole --> ,
			Sun2 --> )					IceAsteroid --> *			
			Sun3 --> !
			
			Nebulas						Dust Clouds					Other
			Nebula1 --> &				LightDustCloud --> <		WormHole --> %
			Nebula2 --> ?				HeavyDustCloud --> >		Fissure --> $
																	Corpse --> #
																	Organian --> ^
																	Pulsar --> ~
*/
const char* MapLines[]=
{
   "Mul_HumanSectorAssault",
   "// Open Space - L",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "1|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "2|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|.........I...................L..........|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................i.l...................|",
   "4|......G...........g.h.............H.....|",
   " |..................k.j...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|.........K...................J..........|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "6|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Open Space - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "1|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "2|.........I.....L.........|",
   " |..........i...l..........|",
   " |.........................|",
   " |......Gg...........hH....|",
   " |.........................|",
   "3|..........k...j..........|",
   " |.........K.....J.........|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "4|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Open Space - S",
   " +____1____2____3+",
   " |...............|",
   " |...............|",
   " |...............|",
   " |...............|",
   "1|...............|",
   " |...............|",
   " |.....Ii.lL.....|",
   " |......Gg.hH....|",
   " |.....Kk.jJ.....|",
   "2|...............|",
   " |...............|",
   " |...............|",
   " |...............|",
   " |...............|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",
   "// Asteroid Field - L",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "1|........[.[.............................|",
   " |.........[..............................|",
   " |........[.[..............[.[............|",
   " |..........................[.............|",
   " |.........................[.[............|",
   "2|........................................|",
   " |..............[.[.......................|",
   " |...............[........................|",
   " |..............[.[.......................|",
   " |........................................|",
   "3|.........I...................L..........|",
   " |........................................|",
   " |........................................|",
   " |.........[.[............................|",
   " |..........[.......i.l....[.[............|",
   "4|......G..[.[......g.h.....[.......H.....|",
   " |..................k.j....[.[............|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|.........K...................J..........|",
   " |........................................|",
   " |........................................|",
   " |...................[.[..................|",
   " |....................[...................|",
   "6|...................[.[..................|",
   " |........................................|",
   " |........................................|",
   " |................................[.[.....|",
   " |......[.[........................[......|",
   "7|.......[........................[.[.....|",
   " |......[.[...............................|",
   " |.................[.[....................|",
   " |..................[.....................|",
   " |.................[.[....................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Asteroid Field - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "1|.....**..................|",
   " |.....**..................|",
   " |.................**......|",
   " |.................**......|",
   " |.........................|",
   "2|.........I.....L.........|",
   " |..........i...l..........|",
   " |.........................|",
   " |......Gg...........hH....|",
   " |.........................|",
   "3|..........k...j..........|",
   " |.........K.....J.........|",
   " |.......................**|",
   " |.......................**|",
   " |.**......................|",
   "4|.**......................|",
   " |.........................|",
   " |...................**....|",
   " |...................**....|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Asteroid Field - S",
   " +____1____2____3+",
   " |...............|",
   " |...............|",
   " |...].......].].|",
   " |..].].......]..|",
   "1|......].]......|",
   " |.......].......|",
   " |.....Ii.lL.....|",
   " |......Gg.hH....|",
   " |.....Kk.jJ.....|",
   "2|..]....].......|",
   " |.].]..].]......|",
   " |...............|",
   " |...].].........|",
   " |....].....]....|",
   "3|.........].]...|",
   " +---------------+",
   "      1    2    3",
   "// Asteroid Belt - L",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |.......]]]]]]]]]]]]..............]].....|",
   " |......]............].....]......]..]....|",
   "1|...]]]..............]...].]]]]]]....]...|",
   " |..]..................].].............]..|",
   " |..]...................]..............]..|",
   " |...].................................]..|",
   " |....]............]]]]]]]]............]..|",
   "2|.....]..........]........]...........]..|",
   " |.....]..........]]]]].....]..........]..|",
   " |.....]].............]......].........]..|",
   " |.......]]]]]]]]]]]]]........]]]]]]]]]...|",
   " |........................................|",
   "3|.........I...................L..........|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................i.l...................|",
   "4|......G...........g.h.............H.....|",
   " |..................k.j...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|.........K...................J..........|",
   " |........................................|",
   " |........................................|",
   " |.............]].........................|",
   " |............]..]...........]............|",
   "6|....]].....]....].........].]...........|",
   " |....].]...]......]......]]...].]]]]]....|",
   " |....]..].]........]]]]]]......].....]...|",
   " |....]...]...........................]...|",
   " |....]...............................]...|",
   "7|....]...........]].................]]...|",
   " |....]......]...]..].....].........].....|",
   " |.....]]...].].]....]...].]]]]]]]]]......|",
   " |.......].]...]......]]].................|",
   " |........]...............................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Asteroid Belt - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "1|.........................|",
   " |.........................|",
   " |.......***********.......|",
   " |......*...........*......|",
   " |.....*.............*.....|",
   "2|....*....I.....L....*....|",
   " |..........i...l......*...|",
   " |......................*..|",
   " |......Gg...........hH..*.|",
   " |......................*..|",
   "3|..........k...j......*...|",
   " |....*....K.....J....*....|",
   " |.....*.............*.....|",
   " |......*...........*......|",
   " |.......***********.......|",
   "4|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Asteroid Belt - S",
   " +____1____2____3+",
   " |[[[[[[[[[[[[[[[|",
   " |[.............[|",
   " |[.............[|",
   " |[.............[|",
   "1|[.............[|",
   " |[.............[|",
   " |.....Ii.lL....[|",
   " |......Gg.hH...[|",
   " |.....Kk.jJ....[|",
   "2|[.............[|",
   " |[.............[|",
   " |[.............[|",
   " |[.............[|",
   " |[.............[|",
   "3|[[[[[[[[[[[[[[[|",
   " +---------------+",
   "      1    2    3",
   "// Nebula - L",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |.&......................................|",
   " |........................................|",
   " |........................................|",
   "1|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "2|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|.........I...................L..........|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................i.l...................|",
   "4|......G...........g.h.............H.....|",
   " |..................k.j...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|.........K...................J..........|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "6|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Nebula - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.&.......................|",
   " |.........................|",
   " |.........................|",
   "1|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "2|.........I.....L.........|",
   " |..........i...l..........|",
   " |.........................|",
   " |......Gg...........hH....|",
   " |.........................|",
   "3|..........k...j..........|",
   " |.........K.....J.........|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "4|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Nebula - S",
   " +____1____2____3+",
   " |...............|",
   " |.&.............|",
   " |...............|",
   " |...............|",
   "1|...............|",
   " |...............|",
   " |.....Ii.lL.....|",
   " |......Gg.hH....|",
   " |.....Kk.jJ.....|",
   "2|...............|",
   " |...............|",
   " |...............|",
   " |...............|",
   " |...............|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",
   "///",
   0
};
