/********************************************************************************************************\

	File Name:	Team1.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "Team1.h"
#include "Team1BaseState.h"
#include "Team1BaseVictory.h"
#include "VictoryCondition.h"
#include "Mul_HumanSectorAssault.h"
#include "Team1Starbase.h"
#include "SectorAssaultData.h"
#include "CommonPlayerShip.h"		// to implement startup damage generically

tClassAllocator<tTeam1> gTeam1Allocator;

char	gBaseRace;

/*

	HELPER NOTE:	The victory condition for this team is specific to this team and only to this team.

*/
void tTeam1::mSetupVictoryCondition()
{
	fVictoryCondition = new tTeam1BaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tTeam1::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kWinner_msg );		break;
	case kDevastatingDefeat:
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kLoser_msg );		break;
	case kUnknownVictoryLevel:
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );	break;
	case kDraw:
	default:
		fMissionInfo->mAddToDebriefingList( fTeam, kDefault_msg );		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tTeam1::mSetupState()
{
	mRegisterState( new tTeam1BaseState() );
	mGotoState( typeid( tTeam1BaseState ) );
}

/*

	HELPER NOTE:	The comm states should include the team they apply for when registered.

*/
void tTeam1::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the team's BPV.
	//
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_G,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kNoMetaShipID, kPlayerNPCBS2 );

*/
void tTeam1::mCreateShipsForTeam()
{
	for (int k = kShip1; k <= kShip1 + fpc.getShipCount(); k++)
	{
		SerializableShipDesc ssd = fpc.getSSD(k - kShip1);

		if (ssd.shipCode[0])
		{
			gBaseRace = ssd.shipCode[0];
			mCreateShip( typeid( tTeam1Starbase ),
					 ssd.shipCode,
					 kStartPosition_G, // + shipScriptDescription.fTeamID,
					 kNoMetaShipID,
					 -1, 0, 0, -1, -1,
					 "The Base", // shipScriptDescription.fCustomName.c_str(),
					 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );
		}
	}

	// frds, planet all made in Team7.cpp
	// defsats made in Team8.cpp
}


void tTeam1::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	If the ships that the player chooses for the team from the multiplayer engine are
					not the ships that team should start with, then leave the function overidden empty
					and create the desired ships in tTeamInfo::mCreateShipsForTeam().  If you would
					like to use the ships chosen by the multiplayer engine, then leave this code
					the way it is.
*/

void tTeam1::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	iMapPosition imp;

	if (mGetTag() == kTagA)
		imp = kStartPosition_G;
	else
		imp = kStartPosition_H;

	mCreateShip( typeid( tCommonPlayerShip ),	// implement startup damage
				 shipScriptDescription.fClassName.c_str(), 
				 imp, // kStartPosition_G + shipScriptDescription.fTeamID,
				 kNoMetaShipID,
				 -1, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );
}

