/********************************************************************************************************\

	File Name:	Team2BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Team1.h"
#include "Team2BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"


//
//	Serialization events
//
tTeam2BaseVictoryState::tTeam2BaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tTeam2BaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tTeam2BaseVictoryState::mBeginState()
{}
void tTeam2BaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the team did once you end the mission with
					a call to mSetGameStatus() for this specific team.  It is not necessary to have every
					victory condition handled here (e.g. kAstoundingVictory...kDevastatingDefeat),
					especially if the victory condition is simply a boolean value (e.g. killed or
					has killed).  Also notice that the example is using the world condition for analysis.
					An example follows.

	eVictoryLevels
	tTeam2BaseVictoryState::mCalcVictoryStatus()
	{
		if ( gWC.mField1() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tTeam2BaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	if ((fChildActor->mGetTeam()->mGetTag() == kTagA && gWC.mDidHomeWin()) ||
		(fChildActor->mGetTeam()->mGetTag() == kTagB && !gWC.mDidHomeWin()))
	{
		return kAstoundingVictory;
	}
	else
	{
		return kDevastatingDefeat;
	}
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tBaseVictoryState::mOnEvent1( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been fField1-ed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetField1( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == kPlayerTeam )
		{
			//
			//	the player fField1-ed, so the player has accomplished the primary objective.
			//
			tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
			if ( team )
			{
				if ( !team->mShipsCurrentlyControlled() )
				{
					gScriptInterface.mSetGameStatus( kPlayerLoses );
				}
			}
		}
	}

*/
void tTeam2BaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( kTeam2 );
	Assert( team );
	if ( !team )
		return;

	if ( !team->mShipsCurrentlyControlled() )
	{
		gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kTeam2 ), kPlayerDestroyed );
		mSetTimedInterval(kTimerSecondsSet, 5, 5.0f);
	}
}

void tTeam2BaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( kTeam2 );
	Assert( team );
	if ( !team )
		return;

	if ( !team->mShipsCurrentlyControlled() )
	{
		gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kTeam2 ), kPlayerCaptured );
		mSetTimedInterval(kTimerSecondsSet, 5, 5.0f);
	}
}

void tTeam2BaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( kTeam2 );
	Assert( team );
	if ( !team )
		return;

	if ( team->mShipsCurrentlyControlled() )
	{
		gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kTeam2 ), kPlayerDisengaged );
		mSetTimedInterval(kTimerSecondsSet, 5, 5.0f);
	}
}
void tTeam2BaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tTeam2BaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tTeam2BaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tTeam2BaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tTeam2BaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tTeam2BaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tTeam2BaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tTeam2BaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tTeam2BaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tTeam2BaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTeam2BaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTeam2BaseVictoryState::mOnFlashingButtonPressed()
{}
void tTeam2BaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tTeam2BaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tTeam2BaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tTeam2BaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tTeam2BaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tTeam2BaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tTeam2BaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//

/*

	HELPER NOTE:	When this event is called, you need to pretend that the team associated with the
					playerNumber has been defeated.  So call what you need to, so that you may determine
					if the game has ended.

*/
void tTeam2BaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tTeam2BaseVictoryState::mOnHostChanged()
{}
void tTeam2BaseVictoryState::mOnTimedInterval( int32 timerID )
{
	gWC.mCheckForGameEnd();
}
void tTeam2BaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tTeam2BaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


