/********************************************************************************************************\

	File Name:	Team5.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "Team5.h"
#include "Team5BaseState.h"
#include "Team5BaseVictory.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"
#include "CommonPlayerShip.h"		// to implement startup damage generically


tClassAllocator<tTeam5> gTeam5Allocator;

/*

	HELPER NOTE:	The victory condition for this team is specific to this team and only to this team.

*/
void tTeam5::mSetupVictoryCondition()
{
	fVictoryCondition = new tTeam5BaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tTeam5::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kWinner_msg );		break;
	case kDevastatingDefeat:
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kLoser_msg );		break;
	case kUnknownVictoryLevel:
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );	break;
	case kDraw:
	default:
		fMissionInfo->mAddToDebriefingList( fTeam, kDefault_msg );		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tTeam5::mSetupState()
{
	mRegisterState( new tTeam5BaseState() );
	mGotoState( typeid( tTeam5BaseState ) );
}

/*

	HELPER NOTE:	The comm states should include the team they apply for when registered.

*/
void tTeam5::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the team's BPV.
	//
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_G,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tTeam5::mCreateShipsForTeam()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the team's BPV.
	//
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_G,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kNoMetaShipID, kPlayerNPCBS2 );

*/
void tTeam5::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	If the ships that the player chooses for the team from the multiplayer engine are
					not the ships that team should start with, then leave the function overidden empty
					and create the desired ships in tTeamInfo::mCreateShipsForTeam().  If you would
					like to use the ships chosen by the multiplayer engine, then leave this code
					the way it is.

*/
void tTeam5::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	iMapPosition imp;

	if (mGetTag() == kTagA)
		imp = kStartPosition_G;
	else
		imp = kStartPosition_H;

	mCreateShip( typeid( tCommonPlayerShip ),	// implement startup damage
				 shipScriptDescription.fClassName.c_str(), 
				 imp, // kStartPosition_G + shipScriptDescription.fTeamID,
				 kNoMetaShipID,
				 -1, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );
}

