/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"

/*

	HELPER NOTE:	This global reference to the world condition is here twofold.  One, it can be easily
					accessed from the all victory states; two, it maintains the consistancy relatively
					easily throughout the script.

*/
class tWorldConditions gWC;

/*

	HELPER NOTE:	Remember to instantiate your world conditions here.

*/
tWorldConditions::tWorldConditions()
{
	didFinality = kFalse;
	gameEnded = kFalse;
	homeWon = kFalse;
}


/*

	HELPER NOTE:	Acts as a sort of victory condition for all of the teams.  This is where you can
					analyze the game state according to the world conditions you have kept.  It is recommended
					that you call ::mSetGameStatus() here, when a victory decision has been made for all teams.

*/
void tWorldConditions::mCheckForGameEnd( void )
{
	int32	homeTeamShipCount = 0;
	int32	awayTeamShipCount = 0;
	int32	team;

	if (!gScriptInterface.mIsHost())
		return;							// not host, doesn't matter
	
	//
	// Check to see if there are any enemy ships left.
	// If none left, player wins.
	//

	for (team = (int)kTeam1; team <= (int)kTeam8; team++)
	{
		tTeamInfo *ti = gScriptInterface.mMission()->mGetTeamHandle((eTeamID)team);
		if (ti)
		{
			tShipIterator	tsi = ti->mGetFirstShip();
			while (*tsi)
			{
				// If ship captured, it'll be on a different team

				tTeamInfo * tci = gScriptInterface.mMission()->mGetTeamHandle((eTeamID)tsi->mGetTeam());
				if (tci)
				{
					if (tci->mGetTag() == kTagA)
						homeTeamShipCount++;
					else
						awayTeamShipCount++;
				}

				tsi++;
			}
		}
	}

	if (awayTeamShipCount <= 0)
	{
		mSetGameEnded(kTrue);
		mSetHomeWon(kTrue);

		// All away ships are gone

		for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
		{
			tTeamInfo * tci = gScriptInterface.mMission()->mGetTeamHandle((eTeamID)team);
			if (tci)
			{
				if (tci->mGetTag() == kTagA)
					gScriptInterface.mSetGameStatus((eTeamID) team, kPlayerWins);
				else
					gScriptInterface.mSetGameStatus((eTeamID) team, kPlayerLoses);
			}
		}
	}

	if (homeTeamShipCount <= 0)
	{
		mSetGameEnded(kTrue);
		mSetHomeWon(kFalse);

		// All home ships are gone

		for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
		{
			tTeamInfo * tci = gScriptInterface.mMission()->mGetTeamHandle((eTeamID)team);
			if (tci)
			{
				if (tci->mGetTag() == kTagB)
					gScriptInterface.mSetGameStatus((eTeamID) team, kPlayerWins);
				else
					gScriptInterface.mSetGameStatus((eTeamID) team, kPlayerLoses);
			}
		}
	}

	if (mGetGameEnded() && didFinality == kFalse)
	{
		didFinality = kTrue;

		FILE *fp = fopen("C:\\SectorAssaultResults.TXT", "w");

		fprintf(fp, "Sector Assault Results\n");
		fprintf(fp, "======================\n\n");
		
		gScriptInterface.mMission()->mDisplayMessage(static_cast<eTeamID>(kTeam1), "Game results dumped to C:\\SectorAssaultResults.TXT");

		for (int team = (int)kTeam1; team <= (int)kTeam8; team++)
		{
			tTeamInfo * tci = gScriptInterface.mMission()->mGetTeamHandle((eTeamID)team);
			if (tci)
			{
				MissingShipList::iterator	msli;

				if (tci->mGetTag() == kTagA)
					fprintf(fp, "Team %d: on Home (Defender) Team\n", (team - kTeam1) + 1);
				else
					fprintf(fp, "Team %d: on Away (Attacker) Team\n", (team - kTeam1) + 1);

				fprintf(fp, "    Ships Left:\n");
				tShipIterator tsi = tci->mGetFirstShip();
				while (*tsi)
				{
					fprintf(fp, "        Ship Name '%s'; Type %s; Endgame Health %2.1f%%\n",
						tsi->mGetCustomName(),
						tsi->mGetClassName(), 100.0f - tsi->mGetDamagePercent());
					if (tsi->mGetTeam() != static_cast<eTeamID>(team))
						fprintf(fp, "        ---->> CAPTURED BY TEAM %d <<----\n", (tsi->mGetTeam() - kTeam1) + 1);
					tsi++;
				}

				fprintf(fp, "    Ships Disengaged:\n");
				for (msli = shipsDisengaged.begin(); msli != shipsDisengaged.end(); msli++)
				{
					if (msli->tid == team)
					{
						fprintf(fp, "        Ship Name '%s'; Type %s; Disengaged with Endgame Health %2.1f%%\n",
							msli->name.c_str(), msli->code.c_str(), msli->health);
					}
				}

				fprintf(fp, "    Ships Destroyed:\n");
				for (msli = shipsDestroyed.begin(); msli != shipsDestroyed.end(); msli++)
				{
					if (msli->tid == team)
					{
						fprintf(fp, "        Ship Name '%s'; Type %s\n", msli->name.c_str(), msli->code.c_str());
					}
				}

				fprintf(fp, "\n");
			}
		}

		fclose(fp);
	}
}

void tWorldConditions::mSetShipDestroyed(eTeamID tid, std::string code, std::string name)
{ 
	char buf[255];
	tTeamInfo * tci = gScriptInterface.mMission()->mGetTeamHandle(tid);
	if (tci->mGetTag() == kTagA)
		sprintf(buf, "Defending Ship Destroyed: %s (%s) on Team %d", code.c_str(), name.c_str(), (tid - kTeam1) + 1);
	else
		sprintf(buf, "Attacking Ship Destroyed: %s (%s) on Team %d", code.c_str(), name.c_str(), (tid - kTeam1) + 1);
	for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
		gScriptInterface.mMission()->mDisplayMessage(static_cast<eTeamID>(team), buf);

	SomeShip ms;

	ms.tid = tid;
	ms.code = code;
	ms.name = name;
	ms.health = 0.0f;
	
	shipsDestroyed.push_back(ms);
}

void tWorldConditions::mSetShipDisengaged(eTeamID tid, std::string code, std::string name, float health)
{ 
	char buf[255];
	tTeamInfo * tci = gScriptInterface.mMission()->mGetTeamHandle(tid);
	if (tci->mGetTag() == kTagA)
		sprintf(buf, "Defending Ship Disengaged: %s (%s) on Team %d", code.c_str(), name.c_str(), (tid - kTeam1) + 1);
	else
		sprintf(buf, "Attacking Ship Disengaged: %s (%s) on Team %d", code.c_str(), name.c_str(), (tid - kTeam1) + 1);
	for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
		gScriptInterface.mMission()->mDisplayMessage(static_cast<eTeamID>(team), buf);

	SomeShip ms;

	ms.tid = tid;
	ms.code = code;
	ms.name = name;
	ms.health = 0.0f;
	
	shipsDisengaged.push_back(ms);
}

void tWorldConditions::mSetShipCaptured(eTeamID tid, std::string code, std::string name)
{ 
	char buf[255];
	tTeamInfo * tci = gScriptInterface.mMission()->mGetTeamHandle(tid);
	if (tci->mGetTag() == kTagA)
		sprintf(buf, "Attacking Ship %s (%s) Captured by Defending Team %d", code.c_str(), name.c_str(), (tid - kTeam1) + 1);
	else
		sprintf(buf, "Defending Ship %s (%s) Captured by Attacking Team %d", code.c_str(), name.c_str(), (tid - kTeam1) + 1);
	for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
		gScriptInterface.mMission()->mDisplayMessage(static_cast<eTeamID>(team), buf);
}

