/********************************************************************************************************\

	File Name:	worldconditions.h
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#if !defined( WorldConditions_h )
#define WorldConditions_h

#include <string>

typedef struct 
{
	eTeamID			tid;
	std::string		code;
	std::string		name;
	float			health;
} SomeShip;

typedef std::vector< SomeShip >	MissingShipList;

class tWorldConditions
{
public:

	tWorldConditions();

	/*

		HELPER NOTE:	Make certain to call this in each and every place the game could possibly end.
						You should try to limit yourself to the teams' victory states though.

	*/
	void mCheckForGameEnd( void );

/*

	HELPER NOTE:	Place your world conditions here.  A world condition is usually a flag or a count
					of some object that determines the state the world is in (e.g. should the game end?,
					should the next team come into play?, etc).

*/
//
//	World condition accessors.
//
public:

	inline void		mSetGameEnded(iTruth it) { gameEnded = it;   };
	inline iTruth	mGetGameEnded(void)      { return gameEnded; };

	inline void		mSetHomeWon(iTruth it)   { homeWon = it;     };
	inline iTruth	mDidHomeWin(void)        { return homeWon;   };

	MissingShipList	mGetShipsDestroyed(void)  { return shipsDestroyed;  }
	MissingShipList	mGetShipsDisengaged(void) { return shipsDisengaged; }

	void			mSetShipDestroyed(eTeamID tid, std::string code, std::string name);
	void			mSetShipDisengaged(eTeamID tid, std::string code, std::string name, float health);
	void			mSetShipCaptured(eTeamID tid, std::string code, std::string name);

//
//	World condition flags.
//
private:

	iTruth	gameEnded;
	iTruth	didFinality;
	iTruth	homeWon;

	MissingShipList	shipsDestroyed;
	MissingShipList shipsDisengaged;
};

extern tWorldConditions gWC;

#endif

