#include "stdafx.h"

#include <math.h>

#include "ShipList.h"
#define _define_vwa
#include "ValidWeaponArcs.h"

static void CompareShips(Ship &s1, Ship &s2);
static void ConsistencyCheck(ShipList &s);

class ShipEditValidHardpoints
{
public:

	char	mod[32];
	int		hp[26];		/* hardpoints 1 through 25 */

	ShipEditValidHardpoints() { };
	ShipEditValidHardpoints(const ShipEditValidHardpoints &copy)
	{
		strcpy(mod, copy.mod);
		for (int i = 0; i < 26; i++)
			hp[i] = copy.hp[i];
	};
	~ShipEditValidHardpoints() { };

};

typedef std::vector<ShipEditValidHardpoints>	SEVAList;
SEVAList	gSEVA;
void LoadShipEditValidHardpoints(void);

int main(int argc, char* argv[])
{
	ShipList	oldList;
	ShipList	newList;

	if (argc != 5)
	{
		printf("usage: Comparator <old shiplist> <old ftrlist> <new shiplist> <new ftrlist>\n", argv[0]);
		exit(1);
	}

	printf("Comparator v1.3\nby MagnumMan-iCoP (James E. King, III)\n\n");
	printf("Redirecting stdout to compare.txt\n");
	freopen( "compare.txt", "w", stdout );
	printf("Comparator v1.3\nby MagnumMan-iCoP (James E. King, III)\n\n");

	LoadShipEditValidHardpoints();
	
	printf("Loading the old spec files (%s, %s)...\n", argv[1], argv[2]);
	if (!oldList.CreateMaster(argv[1], argv[2]))
	{
		printf("error: Could not load old data.\n");
		exit(1);
	}
	
	printf("Loading the new spec files (%s, %s)...\n", argv[3], argv[4]);
	if (!newList.CreateMaster(argv[3], argv[4]))
	{
		printf("error: Could not load new data.\n");
		exit(1);
	}
	ConsistencyCheck(newList);

	printf("\nPerforming detailed regression test...\n");

	/*
	 * iter is an iterator into the list here... so we walk
	 * down the "old ship spec file" looking for changes
	 * in the new one.
	 *
	 * Then we walk down the "new ship spec file" and look
	 * for ships that are missing in the old one - i.e. new
	 * ships.
	 */

	printf("\nScanning for fighters and PFs that have been removed:\n");
	{
		SHIP_LIST_ITER(oldList.fighters_and_pfs)

			Ship	newShip;

			if (!newList.FindFighter(iter->shipCode, newShip))
			{
				printf("  %s (%s) removed.\n", iter->shipCode.c_str(), iter->fullName.c_str());
			}

		SHIP_LIST_ITER_END
	}

	printf("\nScanning for ships that have been removed:\n");
	{
		SHIP_LIST_ITER(oldList)

			Ship	newShip;

			if (!newList.Find(iter->shipCode, newShip))
			{
				printf("  %s (%s) removed.\n", iter->shipCode.c_str(), iter->fullName.c_str());
			}

		SHIP_LIST_ITER_END
	}

	printf("\nScanning for fighters and PFs that have been added:\n");
	{
		SHIP_LIST_ITER(newList.fighters_and_pfs)

			Ship	newShip;

			if (!oldList.FindFighter(iter->shipCode, newShip))
			{
				printf("  %s (%s) added.\n", iter->shipCode.c_str(), iter->fullName.c_str());
			}

		SHIP_LIST_ITER_END
	}

	printf("\nScanning for ships that have been added:\n");
	{
		SHIP_LIST_ITER(newList)

			Ship	newShip;

			if (!oldList.Find(iter->shipCode, newShip))
			{
				printf("  %s (%s) added.\n", iter->shipCode.c_str(), iter->fullName.c_str());
			}

		SHIP_LIST_ITER_END
	}

	printf("\nScanning for fighters and PFs that have changed:\n");
	{
		SHIP_LIST_ITER(oldList.fighters_and_pfs)

			Ship	newShip;

			if (newList.FindFighter(iter->shipCode, newShip))
			{
				CompareShips(*iter, newShip);
			}

		SHIP_LIST_ITER_END
	}
	printf("\nScanning for ships that have changed:\n");
	{
		SHIP_LIST_ITER(oldList)

			Ship	newShip;

			if (newList.Find(iter->shipCode, newShip))
			{
				CompareShips(*iter, newShip);
			}

		SHIP_LIST_ITER_END
	}

	return 0;
}

static char *fo(int val)
{
	static char foo[32];
	sprintf(foo, "%c%d", (val >= 0) ? '+' : '-', abs(val));
	return foo;
}

void LoadShipEditValidHardpoints(void)
{
	char buf[512];

	FILE *fp = fopen("HARDPOINTINFO.TXT", "r");
	if (!fp)
	{
		printf("HARDPOINTINFO.TXT not found in working dir... valid list not initialized.\n\n");
		return;
	}

	printf("Reading ShipEdit's HARDPOINTINFO.TXT file (in working dir)...\n");
	while (fgets(buf, 512, fp))
	{
		ShipEditValidHardpoints seva;

		if (!buf[0]) continue;
		if (isspace(buf[0])) continue;

		sscanf(buf, "%[^,]s", seva.mod);
		
		char *ptr = strchr(buf, ',');
		while (!isdigit(*ptr)) ++ptr;
		sscanf(ptr, "%d,%d,%d,%d,%d, %d,%d,%d,%d,%d, %d,%d,%d,%d,%d, %d,%d,%d,%d,%d, %d,%d,%d,%d,%d",
			&seva.hp[1], &seva.hp[2], &seva.hp[3], &seva.hp[4], &seva.hp[5], 
			&seva.hp[6], &seva.hp[7], &seva.hp[8], &seva.hp[9], &seva.hp[10], 
			&seva.hp[11], &seva.hp[12], &seva.hp[13], &seva.hp[14], &seva.hp[15], 
			&seva.hp[16], &seva.hp[17], &seva.hp[18], &seva.hp[19], &seva.hp[20], 
			&seva.hp[21], &seva.hp[22], &seva.hp[23], &seva.hp[24], &seva.hp[25]);

		gSEVA.push_back(seva);
	}

	fclose(fp);
	printf("ShipEdit Valid Hardpoint list installed.\n\n");
}

static void ConsistencyCheck(ShipList &s)
{
	SHIP_LIST_ITER(s)

		Ship	aShip = *iter;

		//
		// Consistency Checks
		//

		int hasDrones = 0;
		for (int h = kNewFirstWeaponHardPoint; h < kNewLastWeaponHardPoint; h++)
		{
			WeaponMount	s1w = aShip.weapons[(eSystemHardPoint)h];

			if (s1w.weaponType >= kShipSystemHeavyWeaponDroneMinValue &&
				s1w.weaponType <= kShipSystemHeavyWeaponDroneMaxValue)
			{
				hasDrones += s1w.count;
			}
		}
			
		if (hasDrones && aShip.maxDroneControl < hasDrones)
		{
			printf("Warning: Ship %s has %d drones and insufficient drone control (%d).\n", aShip.shipCode.c_str(), hasDrones, aShip.maxDroneControl);
		}
		
		//
		// Valid hardpoint checks
		//

		if (aShip.shipClass != kClassPseudoFighter &&
			aShip.shipClass != kClassShuttle &&
			aShip.shipClass != kClassMonster &&
			aShip.shipClass != kClassListeningPost && 
			aShip.shipClass != kClassPlanets)
		{
			ShipEditValidHardpoints	seva;
			for (SEVAList::iterator it = gSEVA.begin(); it != gSEVA.end(); it++)
			{
				if (!strcmp(aShip.geoUI.c_str(),  it->mod))
				{
					for (int v = kNewFirstWeaponHardPoint; it != gSEVA.end() && v < kNewLastWeaponHardPoint; v++)
					{
						WeaponMount	s1w = aShip.weapons[(eSystemHardPoint)v];

						if (s1w.weaponType != kNoShipSystemType)
						{
							int slot = (v - kNewFirstWeaponHardPoint) + 1;
							if (!it->hp[slot])
							{
								printf("Error: Ship %s has a weapon (%dx%s-%s) in an invalid hardpoint slot (%d) geoUI %s\n",
									aShip.shipCode.c_str(),
									s1w.count, s1w.weaponTypeStr.c_str(), s1w.weaponArcStr.c_str(),
									slot, aShip.geoUI.c_str());
							}
						}
					}

					break;
				}
			}
		}
		
		//
		// Weapons Arc Checks
		//

		for (int g = kNewFirstWeaponHardPoint; g < kNewLastWeaponHardPoint; g++)
		{
			WeaponMount	s1w = aShip.weapons[(eSystemHardPoint)g];
			ValidWeaponArcs	*vwa;

			if ((aShip.shipClass == kClassSpecial && aShip.turnMode <= kTurnModeF) || 
				(aShip.shipClass >= kClassFreighter && aShip.shipClass <= kClassBattleship))
				vwa = vwa_ships;
			else if (aShip.shipClass >= kClassListeningPost)
				vwa = vwa_bases;
			else if (aShip.shipClass <= kClassPseudoFighter)
				vwa = vwa_pfs;
			else
				vwa = 0;

			if (!vwa)
				continue;

			for (int m = 0; vwa[m].weaponType != kNoShipSystemType; m++)
			{
				if (vwa[m].weaponType == s1w.weaponType)
				{
					bool found = false;

					for (int a = 0; strcmp(vwa[m].validArcs[a], "NONE"); a++)
					{
						if (!strcmp(s1w.weaponArcStr.c_str(), vwa[m].validArcs[a]))
						{
							found = true;
							break;
						}
					}

					if (!found)
					{
						printf("Warning: Ship %s has a weapon with an invalid arc: %dx%s-%s\n", aShip.shipCode.c_str(), s1w.count, s1w.weaponTypeStr.c_str(), s1w.weaponArcStr.c_str());
					}
				}
			}
		}

	SHIP_LIST_ITER_END
}

static void CompareShips(Ship &s1, Ship &s2)
{
	char buf[64];
	std::string	changeBuf;
	changeBuf = "";

	if (s1.bpv != s2.bpv)
	{
		sprintf(buf, "%s bpv", fo(s2.bpv - s1.bpv));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.hullType != s2.hullType)
	{
		sprintf(buf, "Hull Type Changed");
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.shipClass != s2.shipClass)
	{
		sprintf(buf, "Ship Class Changed");
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.specialRole != s2.specialRole)
	{
		sprintf(buf, "Special Role Changed");
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.yearFirstAvail != s2.yearFirstAvail)
	{
		sprintf(buf, "%s yfa", fo(s2.yearFirstAvail - s1.yearFirstAvail));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.yearLastAvail != s2.yearLastAvail)
	{
		sprintf(buf, "%s yla", fo(s2.yearLastAvail - s1.yearLastAvail));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.sizeClass != s2.sizeClass)
	{
		sprintf(buf, "Size Class Changed");
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.turnMode != s2.turnMode)
	{
		sprintf(buf, "Turn Mode Changed");
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.moveCost != s2.moveCost)
	{
		sprintf(buf, "%c%01.1f moveCost", (((s2.moveCost - s1.moveCost) > 0) ? '+' : '-'),
										   (((s2.moveCost - s1.moveCost) > 0) ? (s2.moveCost - s1.moveCost) : (s1.moveCost - s2.moveCost)));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.hetPlusTwo != s2.hetPlusTwo)
	{
		sprintf(buf, "%d het+2", fo(s2.hetPlusTwo - s1.hetPlusTwo));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.hetBreakdown != s2.hetBreakdown)
	{
		sprintf(buf, "%s hetBreak", fo(s2.hetBreakdown - s1.hetBreakdown));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.stealthECM != s2.stealthECM)
	{
		sprintf(buf, "%s stealthECM", fo(s2.stealthECM - s1.stealthECM));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.regularCrew != s2.regularCrew)
	{
		sprintf(buf, "%s regularCrew", fo(s2.regularCrew - s1.regularCrew));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.marinesBase != s2.marinesBase)
	{
		sprintf(buf, "%s marinesBase", fo(s2.marinesBase - s1.marinesBase));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.marinesMax != s2.marinesMax)
	{
		sprintf(buf, "%s marinesMax", fo(s2.marinesMax - s1.marinesMax));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.deckCrew != s2.deckCrew)
	{
		sprintf(buf, "%s deckCrew", fo(s2.deckCrew - s1.deckCrew));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.totalCrew != s2.totalCrew)
	{
		sprintf(buf, "%s totalCrew", fo(s2.totalCrew - s1.totalCrew));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.minimumCrew != s2.minimumCrew)
	{
		sprintf(buf, "%s minimumCrew", fo(s2.minimumCrew - s1.minimumCrew));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	for (int k = 0; k < kShieldIndexCount; k++)
	{
		if (s1.shields[k] != s2.shields[k])
		{
			sprintf(buf, "%s shield%d", fo(s2.shields[k] - s1.shields[k]), k+1);
			if (changeBuf.length())
				changeBuf.append(", ");
			changeBuf.append(buf);
		}
	}

	if (s1.totalShielding != s2.totalShielding)
	{
		sprintf(buf, "%s totalShielding", fo(s2.totalShielding - s1.totalShielding));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.cloakCost != s2.cloakCost)
	{
		sprintf(buf, "%s cloakCost", fo(s2.cloakCost - s1.cloakCost));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	for (int h = kNewFirstWeaponHardPoint; h < kNewLastWeaponHardPoint; h++)
	{
		WeaponMount	s1w = s1.weapons[(eSystemHardPoint)h];
		WeaponMount s2w = s2.weapons[(eSystemHardPoint)h];
		
		if (s1w.weaponType != s2w.weaponType ||
			s1w.weaponArc  != s2w.weaponArc ||
			s1w.count      != s2w.count ||
			s1w.shots      != s2w.shots)
		{
			if (!s1w.count)
			{
				sprintf(buf, "Added %dx%s-%s", s2w.count, s2w.weaponTypeStr.c_str(), s2w.weaponArcStr.c_str());
				if (s2w.shots)
					sprintf(buf + strlen(buf), " (%d shots)", s2w.shots);
			}
			else if (!s2w.count)
			{
				sprintf(buf, "Removed %dx%s-%s", s1w.count, s1w.weaponTypeStr.c_str(), s1w.weaponArcStr.c_str());
				if (s1w.shots)
					sprintf(buf + strlen(buf), " (%d shots)", s1w.shots);
			}
			else
			{
				sprintf(buf, "%dx%s-%s", s1w.count, s1w.weaponTypeStr.c_str(), s1w.weaponArcStr.c_str());
				if (s1w.shots)
					sprintf(buf + strlen(buf), " (%d shots)", s1w.shots);
				sprintf(buf + strlen(buf), " -> %dx%s-%s", s2w.count, s2w.weaponTypeStr.c_str(), s2w.weaponArcStr.c_str());
				if (s2w.shots)
					sprintf(buf + strlen(buf), " (%d shots)", s2w.shots);
			}

			if (changeBuf.length())
				changeBuf.append(", ");
			changeBuf.append(buf);
		}
	}
		
	if (s1.probes != s2.probes)
	{
		sprintf(buf, "%s probes", fo((s2.probes - s1.probes) * 5));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.tBombsBase != s2.tBombsBase)
	{
		sprintf(buf, "%s tBombsBase", fo(s2.tBombsBase - s1.tBombsBase));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.tBombsMax != s2.tBombsMax)
	{
		sprintf(buf, "%s tBombsMax", fo(s2.tBombsMax - s1.tBombsMax));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.nukeBombsBase != s2.nukeBombsBase)
	{
		sprintf(buf, "%s nukeBombsBase", fo(s2.nukeBombsBase - s1.nukeBombsBase));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.nukeBombsMax != s2.nukeBombsMax)
	{
		sprintf(buf, "%s nukeBombsMax", fo(s2.nukeBombsMax - s1.nukeBombsMax));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.maxDroneControl != s2.maxDroneControl)
	{
		sprintf(buf, "%s maxDroneControl", fo(s2.maxDroneControl - s1.maxDroneControl));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.add_6 != s2.add_6)
	{
		sprintf(buf, "%s add_6", fo(s2.add_6 - s1.add_6));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.add_12 != s2.add_12)
	{
		sprintf(buf, "%s add_12", fo(s2.add_12 - s1.add_12));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.shuttlesSize != s2.shuttlesSize)
	{
		sprintf(buf, "%s shuttlesSize", fo(s2.shuttlesSize - s1.shuttlesSize));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.launchRate != s2.launchRate)
	{
		sprintf(buf, "%s launchRate", fo(s2.launchRate - s1.launchRate));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.adminBase != s2.adminBase)
	{
		sprintf(buf, "%s adminBase", fo(s2.adminBase - s1.adminBase));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.adminMax != s2.adminMax)
	{
		sprintf(buf, "%s adminMax", fo(s2.adminMax - s1.adminMax));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	///
	if (s1.armor != s2.armor)
	{
		sprintf(buf, "%s armor", fo(s2.armor - s1.armor));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.forwardHull != s2.forwardHull)
	{
		sprintf(buf, "%s forwardHull", fo(s2.forwardHull - s1.forwardHull));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.centerHull != s2.centerHull)
	{
		sprintf(buf, "%s centerHull", fo(s2.centerHull - s1.centerHull));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.aftHull != s2.aftHull)
	{
		sprintf(buf, "%s aftHull", fo(s2.aftHull - s1.aftHull));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.cargo != s2.cargo)
	{
		sprintf(buf, "%s cargo", fo(s2.cargo - s1.cargo));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.barracks != s2.barracks)
	{
		sprintf(buf, "%s barracks", fo(s2.barracks - s1.barracks));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.repair != s2.repair)
	{
		sprintf(buf, "%s repair", fo(s2.repair - s1.repair));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.leftWarp != s2.leftWarp)
	{
		sprintf(buf, "%s leftWarp", fo(s2.leftWarp - s1.leftWarp));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.centerWarp != s2.centerWarp)
	{
		sprintf(buf, "%s centerWarp", fo(s2.centerWarp - s1.centerWarp));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.rightWarp != s2.rightWarp)
	{
		sprintf(buf, "%s rightWarp", fo(s2.rightWarp - s1.rightWarp));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.impulse != s2.impulse)
	{
		sprintf(buf, "%s impulse", fo(s2.impulse - s1.impulse));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.apr != s2.apr)
	{
		sprintf(buf, "%s apr", fo(s2.apr - s1.apr));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.battery != s2.battery)
	{
		sprintf(buf, "%s battery", fo(s2.battery - s1.battery));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.bridge != s2.bridge)
	{
		sprintf(buf, "%s bridge", fo(s2.bridge - s1.bridge));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.security != s2.security)
	{
		sprintf(buf, "%s security", fo(s2.security - s1.security));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.lab != s2.lab)
	{
		sprintf(buf, "%s lab", fo(s2.lab - s1.lab));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.transporters != s2.transporters)
	{
		sprintf(buf, "%s transporters", fo(s2.transporters - s1.transporters));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.mechTractors != s2.mechTractors)
	{
		sprintf(buf, "%s mechTractors", fo(s2.mechTractors - s1.mechTractors));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.specialSensors != s2.specialSensors)
	{
		sprintf(buf, "%s specialSensors", fo(s2.specialSensors - s1.specialSensors));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.sensors != s2.sensors)
	{
		sprintf(buf, "%s sensors", fo(s2.sensors - s1.sensors));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.scanners != s2.scanners)
	{
		sprintf(buf, "%s scanners", fo(s2.scanners - s1.scanners));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.explosionStrength != s2.explosionStrength)
	{
		sprintf(buf, "%s explosionStrength", fo(s2.explosionStrength - s1.explosionStrength));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.acceleration != s2.acceleration)
	{
		sprintf(buf, "%s acceleration", fo(s2.acceleration - s1.acceleration));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.damageControl != s2.damageControl)
	{
		sprintf(buf, "%s damageControl", fo(s2.damageControl - s1.damageControl));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.extraDamage != s2.extraDamage)
	{
		sprintf(buf, "%s extraDamage", fo(s2.extraDamage - s1.extraDamage));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.shipCost != s2.shipCost)
	{
		sprintf(buf, "%s shipCost", fo(s2.shipCost - s1.shipCost));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.baseRefitClass != s2.baseRefitClass)
	{
		sprintf(buf, "changed baseRefitClassto '%s'", s2.baseRefitClass.c_str());
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.geometry != s2.geometry)
	{
		sprintf(buf, "changed geometry '%s'", s2.geometry.c_str());
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.geoUI != s2.geoUI)
	{
		sprintf(buf, "changed geoUI '%s'", s2.geoUI.c_str());
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.fullName != s2.fullName)
	{
		sprintf(buf, "changed fullName '%s'", s2.fullName.c_str());
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

	if (s1.refits != s2.refits)
	{
		sprintf(buf, "changed refits '%s'", s2.refits.c_str());
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	}

/*	if (s1.balance != s2.balance)
	{
		sprintf(buf, "%s bal", fo(s2.balance - s1.balance));
		if (changeBuf.length())
			changeBuf.append(", ");
		changeBuf.append(buf);
	} */

	for (int f = 0; f < 4; f++)
	{
		Fighters s1f = s1.fighters[f];
		Fighters s2f = s2.fighters[f];

		if (s1f.second != s2f.second ||
			s1f.first.shipCode != s2f.first.shipCode)
		{
			sprintf(buf, "fighter change %dx%s -> %dx%s", s1f.second, s1f.first.shipCode.c_str(),
														  s2f.second, s2f.first.shipCode.c_str());
			if (changeBuf.length())
				changeBuf.append(", ");
			changeBuf.append(buf);
		}
	}

	if (changeBuf.length())
		printf("  %s: %s\n", s1.shipCode.c_str(), changeBuf.c_str());
}