// StratGuideHelp.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "ShipList.h"

static char * cv_race(Ship &s)
{
	switch (s.race)
	{
		case kFederation:				return "Federation";
		case kKlingon:					return "Klingon";
		case kRomulan:					return "Romulan";
		case kLyran:					return "Lyran";
		case kHydran:					return "Hydran";
		case kGorn:						return "Gorn";
		case kISC:						return "ISC";
		case kMirak:					return "Mirak";

		case kOrion:					return "Orion";
		case kMonster:					return "Monster";
		case kNeutralRace:				return "Neutral";

		case kOrionOrion:				return "Orion Orion";
		case kOrionKorgath:				return "Orion Korgath";
		case kOrionPrime:				return "Orion Prime";
		case kOrionTigerHeart:			return "Orion Tiger Heart";
		case kOrionBeastRaiders:		return "Orion Beast Raiders";
		case kOrionSyndicate:			return "Orion Syndicate";
		case kOrionWyldeFire:			return "Orion WyldeFire";
		case kOrionCamboro:				return "Orion Camboro";
	}
			
	DebugBreak();
	return "None";
}

static char * cv_hull(Ship &s)
{
	switch (s.shipClass)
	{
		case kClassShuttle:				return "Shuttle";
		case kClassPseudoFighter:		return "Pseudo Fighter";
		case kClassFreighter:			return "Freighter";
		case kClassFrigate:				return "Frigate";
		case kClassDestroyer:			return "Destroyer";
		case kClassWarDestroyer:		return "War Destroyer";
		case kClassLightCruiser:		return "Light Cruiser";
		case kClassHeavyCruiser:		return "Heavy Cruiser";
		case kClassNewHeavyCruiser:		return "New Heavy Cruiser";
		case kClassHeavyBattlecruiser:	return "Heavy Battlecruiser";
		case kClassCarrier:				return "Carrier";
		case kClassDreadnought:			return "Dreadnought";
		case kClassBattleship:			return "Battleship";
		case kClassListeningPost:		return "ListeningPost";
		case kClassBaseStation:			return "Base Station";
		case kClassBattleStation:		return "Battle Station";
		case kClassStarBase:			return "Star Base";
		case kClassMonster:				return "Monster";
		case kClassPlanets:				return "Planets";
		case kClassSpecial:				return "Special";
	}

	DebugBreak();
	return "None";
}

static char * cv_turn(Ship &s)
{
	switch (s.turnMode)
	{
		case kTurnModeAA: return "AA";
		case kTurnModeA: return "A";
		case kTurnModeB: return "B";
		case kTurnModeC: return "C";
		case kTurnModeD: return "D";
		case kTurnModeE: return "E";
		case kTurnModeF: return "F";
		case kTurnModeG: return "G";
		case kTurnModeH: return "H";
	}

	DebugBreak();
	return "None";
}

int main(int argc, char* argv[])
{
	printf("MagnumMan's Strategy Guide Helper Tool v1.0 for Orion Pirates\n\n");

	if (argc != 4)
	{
		fprintf(stderr, "usage: StratGuideHelp <output-filename> <input-shiplist> <input-ftrlist>\n");
		exit(1);
	}

	ShipList	theList;

	printf("Ship List:     %s\n", argv[2]);
	printf("Fighter List:  %s\n", argv[3]);
	printf("Output File:   %s\n", argv[1]);
	printf("\n");

	if (!theList.CreateMaster(argv[2], argv[3]))
	{
		fprintf(stderr, "error: Could not create master list from files specified.\n");
		exit(1);
	}

	FILE *	fp = fopen(argv[1], "w");
	if (!fp)
	{
		fprintf(stderr, "error: Could not create output file for writing.\n");
		exit(1);
	}

	fprintf(fp, "Race\tHull Class\tName\tCode\tYears Available\tBPV\tTurn Mode\tMove Cost\tShield 1\tShield 2-6\tShield 3-5\tShield 4\tShield Total\tT-Bombs\tMissile Conntrol\tWarp\tImpulse\tAPR\tBattery\tTransporters\tTractors\tWeapons / Fighters\n");

	{
		SHIP_LIST_ITER(theList)
		
			if (iter->race == kOrion || iter->race == kNeutralRace || iter->shipClass == kClassPlanets)
				continue;

			fprintf(fp, "%s\t", cv_race(*iter));
			fprintf(fp, "%s\t", cv_hull(*iter));
			fprintf(fp, "%s\t", iter->fullName.c_str());
			fprintf(fp, "%s\t", iter->shipCode.c_str());
			fprintf(fp, "%d", 2263 + iter->yearFirstAvail);
			if (iter->yearLastAvail == 999)
				fprintf(fp, "+\t");
			else
				fprintf(fp, " to %d\t", 2263 + iter->yearLastAvail);

			fprintf(fp, "%d\t", iter->bpv);
			fprintf(fp, "%s\t", cv_turn(*iter));
			fprintf(fp, "%01.2f\t", iter->moveCost);
			fprintf(fp, "%d\t%d\t%d\t%d\t%d\t",
				iter->shields[kForeShieldIndex], iter->shields[kLeftForeShieldIndex],
				iter->shields[kLeftAftShieldIndex], iter->shields[kAftShieldIndex],
				iter->totalShielding);
			if (iter->tBombsBase)
			{
				if (iter->tBombsMax > iter->tBombsBase)
					fprintf(fp, "%d to %d\t", iter->tBombsBase, iter->tBombsMax);
				else
					fprintf(fp, "%d\t", iter->tBombsMax);
			}
			else
				fprintf(fp, "\t");
			if (iter->maxDroneControl)
				fprintf(fp, "%d\t", iter->maxDroneControl);
			else
				fprintf(fp, "\t");
			fprintf(fp, "%d\t%d\t%d\t%d\t%d\t%d\t",
				iter->leftWarp + iter->centerWarp + iter->rightWarp,
				iter->impulse,
				iter->apr,
				iter->battery,
				iter->transporters,
				iter->tractors);

			// now the weapons array

			bool addcomma = false;
			for (int h = kNewFirstWeaponHardPoint; h < kNewLastWeaponHardPoint; h++)
			{
				WeaponMount	wm = iter->weapons[(eSystemHardPoint)h];
				if (wm.count)
				{
					if (addcomma)
						fprintf(fp, ", ");
					else
						addcomma = true;

					fprintf(fp, "%dx%s (%s)", wm.count, wm.weaponTypeStr.c_str(), wm.weaponArcStr.c_str());
				}
			}

			// and fighters onboard

			for (int f = 0; f < 4; f++)
			{
				if (iter->fighters[f].second)
				{
					if (addcomma)
						fprintf(fp, ", ");
					else
						addcomma = true;

					fprintf(fp, "%dx%s", iter->fighters[f].second, iter->fighters[f].first.shipCode.c_str());
				}
			}

			fprintf(fp, "\n");

		SHIP_LIST_ITER_END
	}

	{
		SHIP_LIST_ITER(theList.fighters_and_pfs)
		
			if (iter->race == kOrion || iter->race == kNeutralRace || iter->shipClass == kClassPlanets)
				continue;

			fprintf(fp, "%s\t", cv_race(*iter));
			fprintf(fp, "%s\t", cv_hull(*iter));
			fprintf(fp, "%s\t", iter->fullName.c_str());
			fprintf(fp, "%s\t", iter->shipCode.c_str());
			fprintf(fp, "%d", 2263 + iter->yearFirstAvail);
			if (iter->yearLastAvail == 999)
				fprintf(fp, "+\t");
			else
				fprintf(fp, " to %d\t", 2263 + iter->yearLastAvail);

			fprintf(fp, "%d\t", iter->bpv);
			fprintf(fp, "%s\t", cv_turn(*iter));
			fprintf(fp, "%01.2f\t", iter->moveCost);
			fprintf(fp, "%d\t%d\t%d\t%d\t%d\t",
				iter->shields[kForeShieldIndex], iter->shields[kLeftForeShieldIndex],
				iter->shields[kLeftAftShieldIndex], iter->shields[kAftShieldIndex],
				iter->totalShielding);
			if (iter->tBombsBase)
			{
				if (iter->tBombsMax > iter->tBombsBase)
					fprintf(fp, "%d to %d\t", iter->tBombsBase, iter->tBombsMax);
				else
					fprintf(fp, "%d\t", iter->tBombsMax);
			}
			else
				fprintf(fp, "\t");
			if (iter->maxDroneControl)
				fprintf(fp, "%d\t", iter->maxDroneControl);
			else
				fprintf(fp, "\t");
			fprintf(fp, "%d\t%d\t%d\t%d\t%d\t%d\t",
				iter->leftWarp + iter->centerWarp + iter->rightWarp,
				iter->impulse,
				iter->apr,
				iter->battery,
				iter->transporters,
				iter->tractors);

			// now the weapons array

			bool addcomma = false;
			for (int h = kNewFirstWeaponHardPoint; h < kNewLastWeaponHardPoint; h++)
			{
				WeaponMount	wm = iter->weapons[(eSystemHardPoint)h];
				if (wm.count)
				{
					if (addcomma)
						fprintf(fp, ", ");
					else
						addcomma = true;

					fprintf(fp, "%dx%s (%s)", wm.count, wm.weaponTypeStr.c_str(), wm.weaponArcStr.c_str());
				}
			}

			fprintf(fp, "\n");

		SHIP_LIST_ITER_END
	}

	fclose(fp);

	printf("Processing complete.\n");
	
	return 1;
}

